@@ -1,6 +1,6 @@
 -- merged file : lualibs-extended-merged.lua
 -- parent file : lualibs-extended.lua
--- merge date  : Wed Jan 30 23:25:40 2019
+-- merge date  : Fri Feb 22 23:26:50 2019
 
 do -- begin closure to overcome local limits and interference
 
@@ -359,6 +359,7 @@ local environment={
  formattedfloat=number.formattedfloat,
  stripzero=lpeg.patterns.stripzero,
  stripzeros=lpeg.patterns.stripzeros,
+ FORMAT=string.f9,
 }
 local arguments={ "a1" } 
 setmetatable(arguments,{ __index=function(t,k)
