@@ -1,4 +1,4 @@
-# $Id: TLPSRC.pm 49037 2018-10-31 18:11:57Z karl $
+# $Id: TLPSRC.pm 49434 2018-12-17 18:48:58Z karl $
 # TeXLive::TLPSRC.pm - module for handling tlpsrc files
 # Copyright 2007-2018 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
@@ -12,7 +12,7 @@ use TeXLive::TLUtils;
 use TeXLive::TLPOBJ;
 use TeXLive::TLTREE;
 
-my $svnrev = '$Revision: 49037 $';
+my $svnrev = '$Revision: 49434 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -144,11 +144,9 @@ sub from_file {
       $category = $1;
 
     } elsif ($line =~ /^shortdesc\s*(.*)$/) {
-      # although we would like to do this, hyphen-latin.tlpsrc contains
-      # multiple short/longdesc, entries. Not worth following up.
-      # $shortdesc
-      #   && die "$srcfile:$lineno: second shortdesc not allowed: $line"
-      #          . "(have $shortdesc)\n";
+      $shortdesc
+        && die "$srcfile:$lineno: second shortdesc not allowed: $line"
+               . "(have $shortdesc)\n";
       $shortdesc = $1;
 
     } elsif ($line =~ /^shortdesc$/) {
