@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: install-menu-text.pl 47845 2018-05-26 15:44:25Z karl $
+# $Id: install-menu-text.pl 50297 2019-03-08 23:16:36Z karl $
 # install-menu-txt.pl
 #
 # Copyright 2007-2017 Norbert Preining, Karl Berry
@@ -514,13 +514,13 @@ sub directories_menu
     ($vars{'TEXDIR'} =~ /\/$/ ? 'texmf-dist' : '/texmf-dist');
   if (!$opt_in_place) {
     print <<"EOF";
- <1> TEXDIR:       $vars{'TEXDIR'}
-     support tree: $texmfdir
+ <1> TEXDIR:         $vars{'TEXDIR'}
+     main tree:      $texmfdir
 EOF
   } else {
     print <<"EOF";
-     TEXDIR:       $vars{'TEXDIR'}
-     support tree: $texmfdir
+     TEXDIR:         $vars{'TEXDIR'}
+     main tree:      $texmfdir
 EOF
   }
   if (!$vars{'instopt_portable'}) {
