@@ -69,7 +69,8 @@ proc load_translations {} {
   while {$i < $::argc} {
     set p [lindex $::argv $i]
     incr i
-    if {$p eq "-lang" || $p eq "--lang"} {
+    if {$p eq "-lang" || $p eq "--lang" || $p eq "-gui-lang" || \
+            $p eq "--gui-lang"} {
       if {$i < $::argc} {
         set ::lang [lindex $::argv $i]
         break
@@ -80,6 +81,12 @@ proc load_translations {} {
     } elseif {[string range $p 0 6] eq "--lang="} {
       set ::lang [string range $p 7 end]
       break
+    } elseif {[string range $p 0 9] eq "-gui-lang="} {
+      set ::lang [string range $p 10 end]
+      break
+    } elseif {[string range $p 0 11] eq "--gui-lang="} {
+      set ::lang [string range $p 12 end]
+      break
     }
   }
   unset i
@@ -126,6 +133,7 @@ proc load_translations {} {
         set messcat $f
         break
       } elseif {[string range $ln_f 0 1] eq [string range $::lang 0 1]} {
+        set ::lang [string range $::lang 0 1]
         set maybe $f
       }
     }
@@ -163,10 +171,12 @@ proc load_translations {} {
           if $inmsgstr {
             # empty line signals end of msgstr
             if {$msgstr ne ""} {
-              set msgid [string map {"\\n" "\n"} $msgid]
-              set msgstr [string map {"\\n" "\n"} $msgstr]
-              set msgid [string map {"\\\\" "\\"} $msgid]
-              set msgstr [string map {"\\\\" "\\"} $msgstr]
+              set msgid [string map {{\n} "\n"} $msgid]
+              set msgstr [string map {{\n} "\n"} $msgstr]
+              set msgid [string map {{\\} "\\"} $msgid]
+              set msgstr [string map {{\\} "\\"} $msgstr]
+              set msgid [string map {{\"} "\""} $msgid]
+              set msgstr [string map {{\"} "\""} $msgstr]
               set ::TRANS($msgid) $msgstr
             }
             set msgid ""
@@ -224,10 +234,10 @@ proc yes_no {b} {
   return $ans
 }
 
-if {$::tcl_platform(os) eq "Darwin"} {
-  # avoid warnings 'tar: Failed to set default locale'
-  set ::env(LC_ALL) "en_US.UTF-8"
-}
+# avoid warnings from tar and perl about locale
+set ::env(LC_ALL) "C"
+unset -nocomplain ::env(LANG)
+unset -nocomplain ::env(LANGUAGE)
 
 ### fonts ###
 
@@ -286,7 +296,7 @@ proc mark_sym {mrk} {
     # under windows, these look slightly better than
     # the non-windows selections
     if $mrk {
-      return "\u25C9" ; # 'fisheye'
+      return "\u2714" ; # 'heavy check mark'
     } else {
       return "\u25CB" ; # 'white circle'
     }
